@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+// SPDX-FileCopyrightText: 2019 - 2023 UnionTech Software Technology Co., Ltd.
 //
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
@@ -6,6 +6,7 @@
 #include "private/dguiapplicationhelper_p.h"
 #include "dplatformhandle.h"
 #include <DFontManager>
+#include <DStandardPaths>
 
 #include <QHash>
 #include <QColor>
@@ -22,6 +23,8 @@
 #include <QLockFile>
 #include <QDirIterator>
 #include <QDesktopServices>
+#include <QLibraryInfo>
+#include <QTimer>
 
 #ifdef Q_OS_UNIX
 #include <QDBusError>
@@ -31,6 +34,7 @@
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QProcess>
+#include <DConfig>
 #endif
 #include <QDir>
 #include <QLockFile>
@@ -92,6 +96,12 @@ EnvReplaceGuard::~EnvReplaceGuard()
 
 DGUI_BEGIN_NAMESPACE
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
+using HelperCreator = DGuiApplicationHelper::HelperCreator;
+#else
+using HelperCreator = DGuiApplicationHelper *(*)();
+#endif
+
 #ifdef QT_DEBUG
 Q_LOGGING_CATEGORY(dgAppHelper, "dtk.dguihelper")
 #else
@@ -105,6 +115,11 @@ Q_GLOBAL_STATIC(DFontManager, _globalFM)
 
 #define WINDOW_THEME_KEY "_d_platform_theme"
 
+#define APP_THEME_TYPE "themeType"
+#define DTK_ENABLE_ANIMATIONS "enableDtkAnimations"
+#define DTK_ANIMATIONS_ENV "D_DTK_DISABLE_ANIMATIONS"
+Q_GLOBAL_STATIC_WITH_ARGS(DTK_CORE_NAMESPACE::DConfig, _d_dconfig, ("org.deepin.dtk.preference"));
+
 /*!
  @private
  */
@@ -116,11 +131,17 @@ public:
     {
         // 临时存储一个无效的指针值, 用于此处条件变量的竞争
         if (m_helper.testAndSetRelaxed(nullptr, INVALID_HELPER)) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+            m_helper.storeRelaxed(creator());
+            m_helper.loadRelaxed()->initialize();
+        }
+        return m_helper.loadRelaxed();
+#else
             m_helper.store(creator());
             m_helper.load()->initialize();
         }
-
         return m_helper.load();
+#endif
     }
 
     inline void clear()
@@ -135,7 +156,7 @@ public:
     }
 
     QAtomicPointer<DGuiApplicationHelper> m_helper;
-    static DGuiApplicationHelper::HelperCreator creator;
+    static HelperCreator creator;
 };
 
 class LoadManualServiceWorker : public QThread
@@ -175,14 +196,50 @@ void LoadManualServiceWorker::checkManualServiceWakeUp()
     start();
 }
 
-DGuiApplicationHelper::HelperCreator _DGuiApplicationHelper::creator = _DGuiApplicationHelper::defaultCreator;
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+class Q_DECL_HIDDEN GuiApplicationEventFilter : public QObject
+{
+public:
+    explicit GuiApplicationEventFilter(DGuiApplicationHelperPrivate *transmitter,
+                                       QObject *parent = nullptr)
+        : QObject(parent)
+        , m_transmitter(transmitter)
+    {
+    }
+    virtual bool eventFilter(QObject *watched, QEvent *event) override
+    {
+        if (watched != qApp)
+            return QObject::eventFilter(watched, event);
+
+        switch(event->type()) {
+        case QEvent::ApplicationFontChange: {
+            const QFont font(qGuiApp->font());
+            m_transmitter->q_func()->fontChanged(font);
+        } break;
+        case QEvent::ApplicationPaletteChange: {
+            m_transmitter->onApplicationPaletteChanged();
+            break;
+        }
+        default:
+            break;
+        }
+        return QObject::eventFilter(watched, event);
+    }
+private:
+    DGuiApplicationHelperPrivate *m_transmitter = nullptr;
+};
+#endif
+
+HelperCreator _DGuiApplicationHelper::creator = _DGuiApplicationHelper::defaultCreator;
 Q_GLOBAL_STATIC(_DGuiApplicationHelper, _globalHelper)
 
 int DGuiApplicationHelperPrivate::waitTime = 3000;
 DGuiApplicationHelper::Attributes DGuiApplicationHelperPrivate::attributes = DGuiApplicationHelper::UseInactiveColorGroup;
+static const DGuiApplicationHelper::SizeMode InvalidSizeMode = static_cast<DGuiApplicationHelper::SizeMode>(-1);
 
 DGuiApplicationHelperPrivate::DGuiApplicationHelperPrivate(DGuiApplicationHelper *qq)
     : DObjectPrivate(qq)
+    , explicitSizeMode(InvalidSizeMode)
 {
 
 }
@@ -214,18 +271,15 @@ void DGuiApplicationHelperPrivate::initApplication(QGuiApplication *app)
     // 跟随application销毁
     qAddPostRoutine(staticCleanApplication);
 
-    q->connect(app, &QGuiApplication::paletteChanged, q, [q, this, app] {
-        // 如果用户没有自定义颜色类型, 则应该通知程序的颜色类型发送变化
-        if (Q_LIKELY(!isCustomPalette())) {
-            Q_EMIT q->themeTypeChanged(q->toColorType(app->palette()));
-            Q_EMIT q->applicationPaletteChanged();
-        } else {
-            qWarning() << "DGuiApplicationHelper: Don't use QGuiApplication::setPalette on DTK application.";
-        }
-    });
-
     // 转发程序自己变化的信号
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    app->installEventFilter(new GuiApplicationEventFilter(this, app));
+#else
     q->connect(app, &QGuiApplication::fontChanged, q, &DGuiApplicationHelper::fontChanged);
+    q->connect(app, &QGuiApplication::paletteChanged, q, [this] {
+        onApplicationPaletteChanged();
+    });
+#endif
 
     if (Q_UNLIKELY(!appTheme)) { // 此时说明appTheme可能已经被初始化为了systemtheme
         if (QGuiApplicationPrivate::is_app_running) {
@@ -238,6 +292,9 @@ void DGuiApplicationHelperPrivate::initApplication(QGuiApplication *app)
         // 此时appTheme等价于systemTheme, 可以直接信号链接
         _q_initApplicationTheme();
     }
+
+    systemSizeMode = static_cast<DGuiApplicationHelper::SizeMode>(systemTheme->sizeMode());
+    q->connect(systemTheme, SIGNAL(sizeModeChanged(int)), q, SLOT(_q_sizeModeChanged(int)));
 }
 
 void DGuiApplicationHelperPrivate::staticInitApplication()
@@ -245,7 +302,7 @@ void DGuiApplicationHelperPrivate::staticInitApplication()
     if (!_globalHelper.exists())
         return;
 
-    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load())
+    if (DGuiApplicationHelper *helper = _globalHelper->helper())
         helper->d_func()->initApplication(qGuiApp);
 }
 
@@ -295,10 +352,19 @@ void DGuiApplicationHelperPrivate::_q_initApplicationTheme(bool notifyChange)
     // 监听与程序主题相关的改变
     QObject::connect(appTheme, &DPlatformTheme::themeNameChanged, app, onAppThemeChanged);
     QObject::connect(appTheme, &DPlatformTheme::paletteChanged, app, onAppThemeChanged);
-    QObject::connect(appTheme, &DPlatformTheme::activeColorChanged, app, [this] {
+    QTimer *timer = new QTimer(app);
+    timer->setInterval(100);
+    timer->setSingleShot(true);
+    QObject::connect(timer, &QTimer::timeout, timer, [this] {
         if (!appPalette)
             notifyAppThemeChanged();
     });
+    QObject::connect(appTheme, &DPlatformTheme::activeColorChanged, app, [timer]{
+            timer->start();
+    });
+    QObject::connect(appTheme, &DPlatformTheme::darkActiveColorChanged, app, [timer] {
+            timer->start();
+    });
 
     // appTheme在此之前可能由systemTheme所代替被使用，此时在创建appTheme
     // 并初始化之后，应当发送信号通知程序主题的改变
@@ -310,16 +376,34 @@ void DGuiApplicationHelperPrivate::_q_initApplicationTheme(bool notifyChange)
 void DGuiApplicationHelperPrivate::notifyAppThemeChanged()
 {
     D_Q(DGuiApplicationHelper);
+    notifyAppThemeChangedByEvent();
+    QMetaObject::invokeMethod(q, [q] () {
+        // 通知主题类型发生变化, 此处可能存在误报的行为, 不过不应该对此做额外的约束
+        // 此信号的行为应当等价于 applicationPaletteChanged
+        Q_EMIT q->themeTypeChanged(q->themeType());
+        // 通知调色板对象的改变
+        Q_EMIT q->applicationPaletteChanged();
+    }, Qt::QueuedConnection);
+}
 
+void DGuiApplicationHelperPrivate::notifyAppThemeChangedByEvent()
+{
     QWindowSystemInterfacePrivate::ThemeChangeEvent event(nullptr);
     // 此事件会促使QGuiApplication重新从QPlatformTheme中获取系统级别的QPalette.
     // 而在deepin平台下, 系统级别的QPalette来源自 \a applicationPalette()
     QGuiApplicationPrivate::processThemeChanged(&event);
-    // 通知主题类型发生变化, 此处可能存在误报的行为, 不过不应该对此做额外的约束
-    // 此信号的行为应当等价于 applicationPaletteChanged
-    Q_EMIT q->themeTypeChanged(q->themeType());
-    // 通知调色板对象的改变
-    Q_EMIT q->applicationPaletteChanged();
+}
+
+void DGuiApplicationHelperPrivate::onApplicationPaletteChanged()
+{
+    D_Q(DGuiApplicationHelper);
+    // 如果用户没有自定义颜色类型, 则应该通知程序的颜色类型发送变化
+    if (Q_LIKELY(!isCustomPalette())) {
+        Q_EMIT q->themeTypeChanged(q->toColorType(qGuiApp->palette()));
+        Q_EMIT q->applicationPaletteChanged();
+    } else {
+        qWarning() << "DGuiApplicationHelper: Don't use QGuiApplication::setPalette on DTK application.";
+    }
 }
 
 bool DGuiApplicationHelperPrivate::isCustomPalette() const
@@ -327,6 +411,91 @@ bool DGuiApplicationHelperPrivate::isCustomPalette() const
     return appPalette || paletteType != DGuiApplicationHelper::UnknownType;
 }
 
+void DGuiApplicationHelperPrivate::setPaletteType(DGuiApplicationHelper::ColorType ct, bool emitSignal)
+{
+    if (paletteType == ct)
+        return;
+
+    if (qGuiApp && qGuiApp->testAttribute(Qt::AA_SetPalette))
+        qWarning() << "DGuiApplicationHelper: Plase check 'QGuiApplication::setPalette',"
+                      " Don't use it on DTK application.";
+
+    paletteType = ct;
+
+    if (!emitSignal) {
+        notifyAppThemeChangedByEvent();
+        return;
+    }
+
+    // 如果未固定调色板, 则paletteType的变化可能会导致调色板改变, 应当通知程序更新数据
+    if (!appPalette)
+        notifyAppThemeChanged();
+
+    D_Q(DGuiApplicationHelper);
+    Q_EMIT q->paletteTypeChanged(paletteType);
+}
+
+void DGuiApplicationHelperPrivate::initPaletteType() const
+{
+    DCORE_USE_NAMESPACE
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::DontSaveApplicationTheme))
+        return;
+
+    if (_d_dconfig.exists())
+        return;
+
+    auto applyThemeType = [this](bool emitSignal){
+        int ct = _d_dconfig->value(APP_THEME_TYPE, DGuiApplicationHelper::UnknownType).toInt();
+        if (ct > DGuiApplicationHelper::DarkType || ct < DGuiApplicationHelper::UnknownType)
+            ct = DGuiApplicationHelper::UnknownType;
+
+        const_cast<DGuiApplicationHelperPrivate *>(this)->setPaletteType(DGuiApplicationHelper::ColorType(ct), emitSignal);
+    };
+
+    applyThemeType(false);
+
+    QObject::connect(_d_dconfig, &DConfig::valueChanged, _d_dconfig, [applyThemeType](const QString &key){
+        if (key != APP_THEME_TYPE)
+            return;
+
+        applyThemeType(true);
+    });
+}
+
+void DGuiApplicationHelperPrivate::_q_sizeModeChanged(int mode)
+{
+    D_Q(DGuiApplicationHelper);
+    qCInfo(dgAppHelper) << "Receiving that system size mode is set to [" << static_cast<DGuiApplicationHelper::SizeMode>(mode)
+                        << "], and old system size mode is [" << systemSizeMode << "]";
+
+    const auto oldSizeMode = fetchSizeMode();
+    systemSizeMode = static_cast<DGuiApplicationHelper::SizeMode>(mode);
+    const auto currentSizeMode = fetchSizeMode();
+    if (oldSizeMode != currentSizeMode)
+        Q_EMIT q->sizeModeChanged(currentSizeMode);
+}
+
+DGuiApplicationHelper::SizeMode DGuiApplicationHelperPrivate::fetchSizeMode(bool *isSystemSizeMode) const
+{
+    if (isSystemSizeMode)
+        *isSystemSizeMode = false;
+    // `setSizeMode` > `D_DTK_SIZEMODE` > `systemSizeMode`
+    if (explicitSizeMode != InvalidSizeMode)
+        return explicitSizeMode;
+
+    static const QString envSizeMode(qEnvironmentVariable("D_DTK_SIZEMODE"));
+    if (!envSizeMode.isEmpty()) {
+        bool ok = false;
+        const auto mode = envSizeMode.toInt(&ok);
+        if (ok)
+            return static_cast<DGuiApplicationHelper::SizeMode>(mode);
+    }
+
+    if (isSystemSizeMode)
+        *isSystemSizeMode = true;
+    return systemSizeMode;
+}
+
 /*!
   \class Dtk::Gui::DGuiApplicationHelper
   \inmodule dtkgui
@@ -390,6 +559,7 @@ void DGuiApplicationHelper::initialize()
     d->init();
 }
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
 /*!
   \brief 创建 DGuiApplicationHelper 对象.
 
@@ -407,6 +577,7 @@ void DGuiApplicationHelper::registerInstanceCreator(DGuiApplicationHelper::Helpe
         _globalHelper->clear();
     }
 }
+#endif
 
 inline static int adjustColorValue(int base, qint8 increment, int max = 255)
 {
@@ -428,6 +599,43 @@ DGuiApplicationHelper::~DGuiApplicationHelper()
     _globalHelper->m_helper = nullptr;
 }
 
+static inline QColor adjustHSLColor(const QColor &base, qint8 hueFloat, qint8 saturationFloat,
+                                    qint8 lightnessFloat, qint8 alphaFloat = 0)
+{
+    if (Q_LIKELY(hueFloat || saturationFloat || lightnessFloat || alphaFloat)) {
+        // 按HSL格式调整
+        int H, S, L, A;
+        base.getHsl(&H, &S, &L, &A);
+
+        H = H > 0 ? adjustColorValue(H, hueFloat, 359) : H;
+        S = adjustColorValue(S, saturationFloat);
+        L = adjustColorValue(L, lightnessFloat);
+
+        return QColor::fromHsl(H, S, L, A);
+    }
+
+    return base;
+}
+
+static inline QColor adjustRGBColor(const QColor &base, qint8 redFloat, qint8 greenFloat,
+                                    qint8 blueFloat, qint8 alphaFloat = 0)
+{
+    if (Q_LIKELY(redFloat || greenFloat || blueFloat || alphaFloat)) {
+        // 按RGB格式调整
+        int R, G, B, A;
+        base.getRgb(&R, &G, &B, &A);
+
+        R = adjustColorValue(R, redFloat);
+        G = adjustColorValue(G, greenFloat);
+        B = adjustColorValue(B, blueFloat);
+        A = adjustColorValue(A, alphaFloat);
+
+        return QColor(R, G, B, A);
+    }
+
+    return base;
+}
+
 /*!
   \brief 调整颜色.
 
@@ -447,28 +655,18 @@ QColor DGuiApplicationHelper::adjustColor(const QColor &base,
                                           qint8 hueFloat, qint8 saturationFloat, qint8 lightnessFloat,
                                           qint8 redFloat, qint8 greenFloat, qint8 blueFloat, qint8 alphaFloat)
 {
-    // 按HSL格式调整
-    int H, S, L, A;
-    base.getHsl(&H, &S, &L, &A);
+    if (Q_UNLIKELY(!base.isValid()))
+        return base;
 
-    H = H > 0 ? adjustColorValue(H, hueFloat, 359) : H;
-    S = adjustColorValue(S, saturationFloat);
-    L = adjustColorValue(L, lightnessFloat);
-    A = adjustColorValue(A, alphaFloat);
+    // First do the adjust of spec of this color, Avoid precision loss caused by color spec conversion.
+    if (Q_UNLIKELY(base.spec() == QColor::Hsl)) {
+        QColor newColor = adjustHSLColor(base, hueFloat, saturationFloat, lightnessFloat, alphaFloat);
+        return adjustRGBColor(newColor, redFloat, greenFloat, blueFloat);
+    }
 
-    QColor new_color = QColor::fromHsl(H, S, L, A);
-
-    // 按RGB格式调整
-    int R, G, B;
-    new_color.getRgb(&R, &G, &B);
-
-    R = adjustColorValue(R, redFloat);
-    G = adjustColorValue(G, greenFloat);
-    B = adjustColorValue(B, blueFloat);
-
-    new_color.setRgb(R, G, B, A);
-
-    return new_color;
+    // For other specs, first do RGB adjust
+    QColor newColor = adjustRGBColor(base, redFloat, greenFloat, blueFloat, alphaFloat);
+    return adjustHSLColor(newColor, hueFloat, saturationFloat, lightnessFloat);
 }
 
 /*!
@@ -535,15 +733,15 @@ QColor DGuiApplicationHelper::blendColor(const QColor &substrate, const QColor &
 }
 
 static QColor light_qpalette[QPalette::NColorRoles] {
-    QColor("#414d68"),                  //WindowText
+    QColor(0, 0, 0, 0.7 * 255),         //WindowText
     QColor("#e5e5e5"),                  //Button
     QColor("#e6e6e6"),                  //Light
     QColor("#e5e5e5"),                  //Midlight
     QColor("#e3e3e3"),                  //Dark
     QColor("#e4e4e4"),                  //Mid
-    QColor("#414d68"),                  //Text
+    QColor(0, 0, 0, 0.7 * 255),         //Text
     Qt::black,                          //BrightText
-    QColor("#414d68"),                  //ButtonText
+    QColor(0, 0, 0, 0.7 * 255),         //ButtonText
     Qt::white,                          //Base
     QColor("#f8f8f8"),                  //Window
     QColor(0, 0, 0, 0.05 * 255),        //Shadow
@@ -554,39 +752,39 @@ static QColor light_qpalette[QPalette::NColorRoles] {
     QColor(0, 0, 0, 0.03 * 255),        //AlternateBase
     Qt::white,                          //NoRole
     QColor(255, 255, 255, 0.8 * 255),   //ToolTipBase
-    Qt::black                           //ToolTipText
+    QColor(0, 0, 0, 0.85 * 255)         //ToolTipText
 };
 
 static QColor dark_qpalette[QPalette::NColorRoles] {
-    QColor("#c0c6d4"),                  //WindowText
+    QColor(255, 255, 255, 0.7 * 255),   //WindowText
     QColor("#444444"),                  //Button
     QColor("#484848"),                  //Light
     QColor("#474747"),                  //Midlight
     QColor("#414141"),                  //Dark
     QColor("#434343"),                  //Mid
-    QColor("#c0c6d4"),                  //Text
+    QColor(255, 255, 255, 0.7 * 255),   //Text
     Qt::white,                          //BrightText
-    QColor("#c0c6d4"),                  //ButtonText
+    QColor(255, 255, 255, 0.7 * 255),   //ButtonText
     QColor("#282828"),                  //Base
     QColor("#252525"),                  //Window
     QColor(0, 0, 0, 0.05 * 255),        //Shadow
-    QColor("#0081ff"),                  //Highlight
+    QColor("#024CCA"),                  //Highlight
     QColor("#F1F6FF"),                  //HighlightedText
     QColor("#0082fa"),                  //Link
     QColor("#ad4579"),                  //LinkVisited
     QColor(0, 0, 0, 0.05 * 255),        //AlternateBase
     Qt::black,                          //NoRole
     QColor(45, 45, 45, 0.8 * 255),      //ToolTipBase
-    QColor("#c0c6d4")                   //ToolTipText
+    QColor(255, 255, 255, 0.85 * 255)   //ToolTipText
 };
 
 static QColor light_dpalette[DPalette::NColorTypes] {
     QColor(),                       //NoType
     QColor(0, 0, 0, 255 * 0.03),    //ItemBackground
-    QColor("#001A2E"),              //TextTitle
-    QColor("#526A7F"),              //TextTips
+    QColor(0, 0, 0, 0.85 * 255),    //TextTitle
+    QColor(0, 0, 0, 0.6 * 255),     //TextTips
     QColor("#FF5736"),              //TextWarning
-    Qt::white,                      //TextLively
+    Qt::black,                      //TextLively
     QColor("#0081FF"),              //LightLively
     QColor("#0081FF"),              //DarkLively
     QColor(0, 0, 0, 0.05 * 255),    //FrameBorder
@@ -598,8 +796,8 @@ static QColor light_dpalette[DPalette::NColorTypes] {
 static QColor dark_dpalette[DPalette::NColorTypes] {
     QColor(),                           //NoType
     QColor(255, 255, 255, 255 * 0.05),  //ItemBackground
-    QColor("#C0C6D4"),                  //TextTitle
-    QColor("#6D7C88"),                  //TextTips
+    QColor(255, 255, 255, 0.85 * 255),  //TextTitle
+    QColor(255, 255, 255, 0.6 * 255),   //TextTips
     QColor("#E43F2E"),                  //TextWarning
     Qt::white,                          //TextLively
     QColor("#0059d2"),                  //LightLively
@@ -677,9 +875,7 @@ DPalette DGuiApplicationHelper::standardPalette(DGuiApplicationHelper::ColorType
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
         if (role == QPalette::PlaceholderText) {
-            // 5.15新添加此颜色 这里使用5.11的颜色保证效果与5.11对齐
             color = dcolor_list[DPalette::PlaceholderText];
-            continue;
         }
 #endif
         // 处理半透明色
@@ -793,15 +989,7 @@ void DGuiApplicationHelper::generatePaletteColor(DPalette &base, QPalette::Color
         base.setBrush(QPalette::Disabled, role, window);
         base.setBrush(QPalette::Inactive, role, window);
         return;
-    } else if (role == QPalette::Highlight && toColorType(base) == DarkType) {
-        // 暗色模式下的高亮色亮度要降低10%，避免太突兀
-        QColor highlight = base.highlight().color();
-
-        if (highlight.isValid()) {
-            base.setColor(QPalette::Highlight, adjustColor(highlight, 0, 0, -20, 0, 0, 0, 0));
-        }
     }
-
     generatePaletteColor_helper(base, role, type);
 }
 
@@ -854,6 +1042,20 @@ void DGuiApplicationHelper::generatePalette(DPalette &base, ColorType type)
     }
 }
 
+static inline QColor getActiveColor(const DPlatformTheme *theme, DGuiApplicationHelper::ColorType type)
+{
+    QColor activeColor;
+    if (type == DGuiApplicationHelper::DarkType) {
+        activeColor = theme->darkActiveColor();
+        if (!activeColor.isValid()) {
+            activeColor = theme->activeColor();
+        }
+    } else {
+        activeColor = theme->activeColor();
+    }
+
+    return activeColor;
+}
 /*!
   \brief 获取调色板数据.
 
@@ -879,7 +1081,7 @@ DPalette DGuiApplicationHelper::fetchPalette(const DPlatformTheme *theme)
 
     bool ok = false;
     base_palette = theme->fetchPalette(standardPalette(type), &ok);
-    const QColor &active_color = theme->activeColor();
+    const QColor &active_color = getActiveColor(theme, type);
 
     if (active_color.isValid()) {
         base_palette.setColor(QPalette::Normal, QPalette::Highlight, active_color);
@@ -897,6 +1099,7 @@ DPalette DGuiApplicationHelper::fetchPalette(const DPlatformTheme *theme)
     return base_palette;
 }
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
 /*!
   \brief 设置是否将调色板的颜色改为半透明模式.
 
@@ -917,6 +1120,7 @@ void DGuiApplicationHelper::setColorCompositingEnabled(bool on)
 {
     DGuiApplicationHelper::setAttribute(Attribute::ColorCompositing, on);
 }
+#endif
 
 bool DGuiApplicationHelper::isXWindowPlatform()
 {
@@ -978,6 +1182,7 @@ DPlatformTheme *DGuiApplicationHelper::applicationTheme() const
     return d->appTheme;
 }
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
 /*!
   \brief 返回窗口级别的主题, 优先级高于 windowTheme 和 systemTheme.
 
@@ -999,6 +1204,7 @@ DPlatformTheme *DGuiApplicationHelper::windowTheme(QWindow *window) const
 
     return theme;
 }
+#endif
 
 /*!
   \brief 返回应用程序调色板.
@@ -1015,7 +1221,7 @@ DPlatformTheme *DGuiApplicationHelper::windowTheme(QWindow *window) const
   \warning 不应该在DTK程序中使用QGuiApplication/QApplication::setPalette
   \return 应用程序调色板
  */
-DPalette DGuiApplicationHelper::applicationPalette() const
+DPalette DGuiApplicationHelper::applicationPalette(ColorType paletteType) const
 {
     D_DC(DGuiApplicationHelper);
 
@@ -1023,7 +1229,7 @@ DPalette DGuiApplicationHelper::applicationPalette() const
         return *d->appPalette;
     }
 
-    ColorType type = d->paletteType;
+    ColorType type = paletteType;
     bool aa_setPalette = qGuiApp && qGuiApp->testAttribute(Qt::AA_SetPalette);
     // 此时appTheme可能还未初始化, 因此先使用systemTheme, 待appTheme初始化之后会
     // 通知程序调色板发生改变
@@ -1046,7 +1252,7 @@ DPalette DGuiApplicationHelper::applicationPalette() const
         // 覆盖DPalette中的的QPalette数据
         pa.QPalette::operator =(qGuiApp->palette());
     } else {
-        const QColor &active_color = theme->activeColor();
+        const QColor &active_color = getActiveColor(theme, type);
 
         if (active_color.isValid()) {
             // 应用Active Color
@@ -1058,6 +1264,13 @@ DPalette DGuiApplicationHelper::applicationPalette() const
     return pa;
 }
 
+DPalette DGuiApplicationHelper::applicationPalette() const
+{
+    D_DC(DGuiApplicationHelper);
+
+    return applicationPalette(d->paletteType);
+}
+
 /*!
   \brief DGuiApplicationHelper::setApplicationPalette.
 
@@ -1079,13 +1292,21 @@ void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
         qWarning() << "DGuiApplicationHelper: Plase check 'QGuiApplication::setPalette', Don't use it on DTK application.";
     }
 
+    auto resolve = [](const DPalette &palette) ->bool {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    return palette.resolveMask();
+#else
+    return palette.resolve();
+#endif
+    };
+
     if (d->appPalette) {
-        if (palette.resolve()) {
+        if (resolve(palette)) {
             *d->appPalette = palette;
         } else {
             d->appPalette.reset();
         }
-    } else if (palette.resolve()) {
+    } else if (resolve(palette)) {
         d->appPalette.reset(new DPalette(palette));
     } else {
         return;
@@ -1095,6 +1316,7 @@ void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
     d->notifyAppThemeChanged();
 }
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
 /*!
   \brief DGuiApplicationHelper::windowPalette.
 
@@ -1127,6 +1349,7 @@ DPalette DGuiApplicationHelper::windowPalette(QWindow *window) const
 
     return fetchPalette(theme);
 }
+#endif
 
 /*!
   \brief DGuiApplicationHelper::fontManager.
@@ -1139,6 +1362,8 @@ const DFontManager *DGuiApplicationHelper::fontManager() const
 {
     // 为对象初始化信号链接
     if (!_globalFM.exists()) {
+        // fontManager's base font is update from qGuiApp.
+        _globalFM->setBaseFont(qGuiApp->font());
         connect(this, &DGuiApplicationHelper::fontChanged, _globalFM, &DFontManager::setBaseFont);
     }
 
@@ -1181,7 +1406,7 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QColor
  */
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QPalette &palette)
 {
-    return toColorType(palette.background().color());
+    return toColorType(palette.window().color());
 }
 
 /*!
@@ -1199,6 +1424,7 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::themeType() const
 {
     D_DC(DGuiApplicationHelper);
 
+    d->initPaletteType();
     if (d->paletteType != UnknownType) {
         return d->paletteType;
     }
@@ -1218,6 +1444,8 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::themeType() const
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::paletteType() const
 {
     D_DC(DGuiApplicationHelper);
+
+    d->initPaletteType();
     return d->paletteType;
 }
 
@@ -1333,8 +1561,8 @@ bool DGuiApplicationHelper::setSingleInstance(const QString &key, DGuiApplicatio
                 qCInfo(dgAppHelper) << "New instance: pid=" << pid << "arguments=" << arguments;
 
                 // 通知新进程的信息
-                if (_globalHelper.exists() && _globalHelper->m_helper.load())
-                    Q_EMIT _globalHelper->m_helper.load()->newProcessInstance(pid, arguments);
+                if (_globalHelper.exists() && _globalHelper->helper())
+                    Q_EMIT _globalHelper->helper()->newProcessInstance(pid, arguments);
             });
 
             instance->flush(); //发送数据给新的实例
@@ -1357,6 +1585,7 @@ void DGuiApplicationHelper::setSingleInstanceInterval(int interval)
     DGuiApplicationHelperPrivate::waitTime = interval;
 }
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
 /*!
   \brief 设置从QLocalServer获取消息的等待时间.
 
@@ -1366,6 +1595,34 @@ void DGuiApplicationHelper::setSingelInstanceInterval(int interval)
 {
     DGuiApplicationHelperPrivate::waitTime = interval;
 }
+#endif
+
+/*!
+  \brief 获取帮助手册目录
+
+  \return 帮助手册目录
+ */
+QStringList DGuiApplicationHelper::userManualPaths(const QString &appName)
+{
+    Q_ASSERT(!appName.isEmpty());
+
+    // 获取主目录
+    DCORE_USE_NAMESPACE
+    QStringList manualPaths;
+    const auto &pathlist = DStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
+
+    // 获取所有已存在的帮助手册目录
+    for (int i = 0; i < pathlist.size(); ++i) {
+        QString strAssetPath = QStringList{pathlist[i], "deepin-manual/manual-assets"}.join(QDir::separator());
+        for (const QString &type : QDir(strAssetPath).entryList(QDir::NoDotAndDotDot | QDir::Dirs)) {
+            QString appDirPath = QStringList{strAssetPath, type, appName}.join(QDir::separator());
+            if (QDir(appDirPath).exists())
+                manualPaths.push_back(appDirPath);
+        }
+    }
+
+    return manualPaths;
+}
 
 /*!
  * \brief Determine whether it's a user manual for this application.
@@ -1373,62 +1630,7 @@ void DGuiApplicationHelper::setSingelInstanceInterval(int interval)
  */
 bool DGuiApplicationHelper::hasUserManual() const
 {
-#ifdef Q_OS_LINUX
-    auto loadManualFromLocalFile = [=]() -> bool {
-        const QString appName = qApp->applicationName();
-        bool dmanBinaryExists = false;
-        bool dmanDataExists = false;
-        const QString sysPath = qgetenv("PATH");
-        auto binPath = sysPath.split(":");
-        for (const auto path : binPath) {
-            if (QFile::exists(QStringList {path, "dman"}.join(QDir::separator()))) {
-                dmanBinaryExists = true;
-                break;
-            }
-        }
-
-        // search all subdirectories
-        const QString xdgDataPath = qgetenv("XDG_DATA_DIRS");
-        auto dataPath = xdgDataPath.split(":");
-        for (const auto path : dataPath) {
-            QString strManualPath = QStringList {path, "deepin-manual"}.join(QDir::separator());
-
-            QDirIterator it(strManualPath, QDirIterator::Subdirectories);
-            while (it.hasNext()) {
-                QFileInfo file(it.next());
-                if (file.isDir() && file.fileName().contains(appName, Qt::CaseInsensitive)) {
-                    dmanDataExists = true;
-                    break;
-                }
-
-                if (file.isDir())
-                    continue;
-            }
-        }
-
-        return  dmanBinaryExists && dmanDataExists;
-    };
-
-    QDBusConnection conn = QDBusConnection::sessionBus();
-    if (conn.interface()->isServiceRegistered("com.deepin.Manual.Search")) {
-        QDBusInterface manualSearch("com.deepin.Manual.Search",
-                                    "/com/deepin/Manual/Search",
-                                    "com.deepin.Manual.Search");
-        if (manualSearch.isValid()) {
-            QDBusReply<bool> reply = manualSearch.call("ManualExists", qApp->applicationName());
-            return reply.value();
-        } else {
-            return loadManualFromLocalFile();
-        }
-    } else {
-        static LoadManualServiceWorker *manualWorker = new LoadManualServiceWorker;
-        manualWorker->checkManualServiceWakeUp();
-
-        return loadManualFromLocalFile();
-    }
-#else
-    return false;
-#endif
+    return userManualPaths(qApp->applicationName()).size() > 0;
 }
 
 bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<QString> &translateDirs, const QList<QLocale> &localeFallback)
@@ -1446,7 +1648,12 @@ bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<
     QStringList missingQmfiles;
     for (const auto &locale : localeFallback) {
         QStringList translateFilenames {QString("%1_%2").arg(fileName).arg(locale.name())};
-        const QStringList parseLocalNameList = locale.name().split("_", QString::SkipEmptyParts);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+        auto behavior = Qt::SkipEmptyParts;
+#else
+        auto behavior = QString::SkipEmptyParts;
+#endif
+        const QStringList parseLocalNameList = locale.name().split("_", behavior);
         if (parseLocalNameList.length() > 0)
             translateFilenames << QString("%1_%2").arg(fileName).arg(parseLocalNameList.at(0));
 
@@ -1455,7 +1662,7 @@ bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<
                 DPathBuf path(dir);
                 QString translatePath = (path / translateFilename).toString();
                 if (QFile::exists(translatePath + ".qm")) {
-                    qDebug() << "load translate" << translatePath;
+                    qCDebug(dgAppHelper) << "load translate" << translatePath;
                     auto translator = new QTranslator(qApp);
                     translator->load(translatePath);
                     qApp->installTranslator(translator);
@@ -1476,6 +1683,63 @@ bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<
     return false;
 }
 
+bool DGuiApplicationHelper::loadTranslator(const QList<QLocale> &localeFallback)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    auto qTranslationsPath = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+#else
+    auto qTranslationsPath = QLibraryInfo::path(QLibraryInfo::TranslationsPath);
+#endif
+
+    DCORE_USE_NAMESPACE
+    //("/home/user/.local/share", "/usr/local/share", "/usr/share")
+    auto dataDirs = DStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
+    QList<QString> qtranslateDirs = { qTranslationsPath };
+    for (const auto &path : dataDirs) {
+        DPathBuf pathBuf(path);
+        qtranslateDirs << (pathBuf / "qt" QT_STRINGIFY(QT_VERSION_MAJOR) / "translations").toString();
+    }
+
+    loadTranslator("qt", qtranslateDirs, localeFallback);
+    loadTranslator("qtbase", qtranslateDirs, localeFallback);
+
+    QList<QString> translateDirs;
+    auto appName = qApp->applicationName();
+    for (const auto &path : dataDirs) {
+        DPathBuf pathBuf(path);
+        translateDirs << (pathBuf / appName / "translations").toString();
+    }
+
+    // ${translateDir}/${appName}_${localeName}.qm
+    return loadTranslator(appName, translateDirs, localeFallback);
+}
+
+DGuiApplicationHelper::SizeMode DGuiApplicationHelper::sizeMode() const
+{
+    D_DC(DGuiApplicationHelper);
+    return d->fetchSizeMode();
+}
+
+void DGuiApplicationHelper::setSizeMode(const DGuiApplicationHelper::SizeMode mode)
+{
+    D_D(DGuiApplicationHelper);
+    const auto old = d->fetchSizeMode();
+    d->explicitSizeMode = mode;
+    const auto current = d->fetchSizeMode();
+    if (old != current)
+        Q_EMIT sizeModeChanged(current);
+}
+
+void DGuiApplicationHelper::resetSizeMode()
+{
+    D_D(DGuiApplicationHelper);
+    const auto old = d->fetchSizeMode();
+    d->explicitSizeMode = InvalidSizeMode;
+    const auto current = d->fetchSizeMode();
+    if (current != old)
+        Q_EMIT sizeModeChanged(current);
+}
+
 void DGuiApplicationHelper::setAttribute(DGuiApplicationHelper::Attribute attribute, bool enable)
 {
     if (attribute < Attribute::ReadOnlyLimit) {
@@ -1501,16 +1765,31 @@ bool DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::Attribute attri
             return false;
         }
         return QString(typeid(*QGuiApplicationPrivate::platform_theme).name()).contains("QDeepinTheme");
-    case IsDeepinEnvironment:
-        return QGuiApplicationPrivate::instance()->platformIntegration()->services()->desktopEnvironment().toLower().contains("deepin");
+    case IsDeepinEnvironment: {
+        const auto &de = QGuiApplicationPrivate::instance()->platformIntegration()->services()->desktopEnvironment();
+        return de.toLower().contains("deepin") || de == "DDE";
+    }
     case IsSpecialEffectsEnvironment: {
         return qgetenv("DTK_DISABLED_SPECIAL_EFFECTS").toInt() != 1;
     }
+    case IsWaylandPlatform: {
+        static bool isTreeland = qApp->platformName() == QByteArrayLiteral("wayland");
+        return isTreeland;
+    }
+    case HasAnimations: {
+        static bool isDisable = qEnvironmentVariableIsSet(DTK_ANIMATIONS_ENV);
+        if (isDisable)
+            return false;
+
+        static bool shouldEnable = _d_dconfig->value(DTK_ENABLE_ANIMATIONS, false).toBool();
+        return shouldEnable;
+    }
     default:
         return DGuiApplicationHelperPrivate::attributes.testFlag(attribute);
     }
 }
 
+#if DTK_VERSION < DTK_VERSION_CHECK(6, 0, 0, 0)
 /*!
   \brief DGuiApplicationHelper::setThemeType.
   \deprecated 同 setPaletteType， 已废弃，请不要再使用。
@@ -1520,6 +1799,7 @@ void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeT
 {
     setPaletteType(themeType);
 }
+#endif
 
 /*!
   \brief 设置程序所应用的调色板类型.
@@ -1534,21 +1814,16 @@ void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeT
  */
 void DGuiApplicationHelper::setPaletteType(DGuiApplicationHelper::ColorType paletteType)
 {
-    D_D(DGuiApplicationHelper);
-
-    if (d->paletteType == paletteType)
+    if (!qApp) {
+        qWarning() << "Can't call `DGuiApplicationHelper::setPaletteType` before QCoreApplication constructed.";
         return;
-
-    if (qGuiApp && qGuiApp->testAttribute(Qt::AA_SetPalette)) {
-        qWarning() << "DGuiApplicationHelper: Plase check 'QGuiApplication::setPalette', Don't use it on DTK application.";
     }
 
-    d->paletteType = paletteType;
-    // 如果未固定调色板, 则paletteType的变化可能会导致调色板改变, 应当通知程序更新数据
-    if (!d->appPalette)
-        d->notifyAppThemeChanged();
+    D_D(DGuiApplicationHelper);
 
-    Q_EMIT paletteTypeChanged(paletteType);
+    d->initPaletteType();
+    d->setPaletteType(paletteType, true);
+    _d_dconfig->setValue(APP_THEME_TYPE, paletteType);
 }
 
 /*!
@@ -1566,12 +1841,19 @@ void DGuiApplicationHelper::handleHelpAction()
     QDBusInterface manual("com.deepin.Manual.Open",
                           "/com/deepin/Manual/Open",
                           "com.deepin.Manual.Open");
-    QDBusReply<void> reply = manual.call("ShowManual", appid);
-    if (reply.isValid())  {
-        return;
-    }
-    // fallback to old interface
-    QProcess::startDetached("dman", QStringList() << appid);
+    QDBusPendingCall call = manual.asyncCall("ShowManual", appid);
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, [appid](QDBusPendingCallWatcher *pWatcher) {
+        QDBusPendingReply<void> reply = *pWatcher;
+        if (reply.isError()) {
+            // fallback to old interface
+            qWarning() << reply.error() << "fallback to dman appid";
+            QProcess::startDetached("dman", QStringList() << appid);
+        }
+
+        pWatcher->deleteLater();
+    });
+
 #else
     qWarning() << "not support dman now";
 #endif
