@@ -1,23 +1,6 @@
-/*
- * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
- *
- * Author:     zccrs <zccrs@live.com>
- *
- * Maintainer: zccrs <zhangjide@deepin.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
 
 #include "dwindowmanagerhelper.h"
 #include "dforeignwindow.h"
@@ -26,12 +9,25 @@
 #include <DGuiApplicationHelper>
 #include <QGuiApplication>
 
-#include <QtPlatformHeaders/QXcbWindowFunctions>
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    #include <qpa/qplatformwindow_p.h>
+    #define D_XCB_WINDOW_TYPE QNativeInterface::Private::QXcbWindow::WindowType
+#else
+    #include <QtPlatformHeaders/QXcbWindowFunctions>
+    #define D_XCB_WINDOW_TYPE QXcbWindowFunctions::WmWindowType
+#endif
+
+
 #include <qpa/qplatformwindow.h>
 #include <qpa/qplatformnativeinterface.h>
 
 #include <functional>
 
+#ifndef DTK_DISABLE_TREELAND
+#include "plugins/platform/treeland/dtreelandwindowmanagerhelper.h"
+#endif
+
 DGUI_BEGIN_NAMESPACE
 
 #define DEFINE_CONST_CHAR(Name) const char _##Name[] = "_d_" #Name
@@ -61,81 +57,53 @@ DEFINE_CONST_CHAR(connectWindowMotifWMHintsChanged);
 DEFINE_CONST_CHAR(popupSystemWindowMenu);
 DEFINE_CONST_CHAR(setWMClassName);
 
-static bool connectWindowManagerChangedSignal(QObject *object, std::function<void ()> slot)
+template<typename ReturnT, typename FunctionT, typename... Args>
+static inline ReturnT callPlatformFunction(const QByteArray &funcName,  Args... args)
 {
-    QFunctionPointer connectWindowManagerChangedSignal = Q_NULLPTR;
+    QFunctionPointer func = Q_NULLPTR;
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectWindowManagerChangedSignal = qApp->platformFunction(_connectWindowManagerChangedSignal);
+    func = qApp->platformFunction(funcName);
 #endif
 
-    return connectWindowManagerChangedSignal && reinterpret_cast<bool(*)(QObject *object, std::function<void ()>)>(connectWindowManagerChangedSignal)(object, slot);
+    return func ? reinterpret_cast<FunctionT>(func)(args...) : ReturnT();
+}
+
+typedef bool(*func_t)(QObject *object, std::function<void ()>);
+static bool connectWindowManagerChangedSignal(QObject *object, std::function<void ()> slot)
+{
+    return callPlatformFunction<bool, func_t>(_connectWindowManagerChangedSignal, object, slot);
 }
 
 static bool connectHasBlurWindowChanged(QObject *object, std::function<void ()> slot)
 {
-    QFunctionPointer connectHasBlurWindowChanged = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectHasBlurWindowChanged = qApp->platformFunction(_connectHasBlurWindowChanged);
-#endif
-
-    return connectHasBlurWindowChanged && reinterpret_cast<bool(*)(QObject *object, std::function<void ()>)>(connectHasBlurWindowChanged)(object, slot);
+    return callPlatformFunction<bool, func_t>(_connectHasBlurWindowChanged, object, slot);
 }
 
 static bool connectHasCompositeChanged(QObject *object, std::function<void ()> slot)
 {
-    QFunctionPointer connectHasCompositeChanged = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectHasCompositeChanged = qApp->platformFunction(_connectHasCompositeChanged);
-#endif
-
-    return connectHasCompositeChanged && reinterpret_cast<bool(*)(QObject *object, std::function<void ()>)>(connectHasCompositeChanged)(object, slot);
+    return callPlatformFunction<bool, func_t>(_connectHasCompositeChanged, object, slot);
 }
 
 static bool connectHasNoTitlebarChanged(QObject *object, std::function<void ()> slot)
 {
-    QFunctionPointer connectHasNoTitlebarChanged = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectHasNoTitlebarChanged = qApp->platformFunction(_connectHasNoTitlebarChanged);
-#endif
-
-    return connectHasNoTitlebarChanged && reinterpret_cast<bool(*)(QObject *object, std::function<void ()>)>(connectHasNoTitlebarChanged)(object, slot);
+    return callPlatformFunction<bool, func_t>(_connectHasNoTitlebarChanged, object, slot);
 }
 
 static bool connectHasWallpaperEffectChanged(QObject *object, std::function<void ()> slot)
 {
-    QFunctionPointer connectHasWallpaperEffectChanged = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectHasWallpaperEffectChanged = qApp->platformFunction(_connectHasWallpaperEffectChanged);
-#endif
-
-    return connectHasWallpaperEffectChanged && reinterpret_cast<bool(*)(QObject *object, std::function<void ()>)>(connectHasWallpaperEffectChanged)(object, slot);
+    return callPlatformFunction<bool, func_t>(_connectHasWallpaperEffectChanged, object, slot);
 }
 
 static bool connectWindowListChanged(QObject *object, std::function<void ()> slot)
 {
-    QFunctionPointer connectWindowListChanged = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectWindowListChanged = qApp->platformFunction(_connectWindowListChanged);
-#endif
-
-    return connectWindowListChanged && reinterpret_cast<bool(*)(QObject *object, std::function<void ()>)>(connectWindowListChanged)(object, slot);
+    return callPlatformFunction<bool, func_t>(_connectWindowListChanged, object, slot);
 }
 
 static bool connectWindowMotifWMHintsChanged(QObject *object, std::function<void (quint32)> slot)
 {
-    QFunctionPointer connectWindowMotifWMHintsChanged = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    connectWindowMotifWMHintsChanged = qApp->platformFunction(_connectWindowMotifWMHintsChanged);
-#endif
-
-    return connectWindowMotifWMHintsChanged && reinterpret_cast<bool(*)(QObject *object, std::function<void (quint32)>)>(connectWindowMotifWMHintsChanged)(object, slot);
+    typedef bool(*quint32_func_t)(QObject *object, std::function<void (quint32)>);
+    return callPlatformFunction<bool, quint32_func_t>(_connectWindowMotifWMHintsChanged, object, slot);
 }
 
 class DWindowManagerHelperPrivate : public DTK_CORE_NAMESPACE::DObjectPrivate
@@ -147,6 +115,11 @@ public:
     mutable QList<DForeignWindow *> windowList;
 };
 
+// TODO abstract an interface to adapt to various WM.
+#ifndef DTK_DISABLE_TREELAND
+Q_GLOBAL_STATIC(TreelandWindowManagerHelper, treelandWMHGlobal)
+#endif
+
 class DWindowManagerHelper_ : public DWindowManagerHelper {};
 Q_GLOBAL_STATIC(DWindowManagerHelper_, wmhGlobal)
 
@@ -359,6 +332,11 @@ DWindowManagerHelper::~DWindowManagerHelper()
  */
 DWindowManagerHelper *DWindowManagerHelper::instance()
 {
+#ifndef DTK_DISABLE_TREELAND
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+        return treelandWMHGlobal;
+    }
+#endif
     return wmhGlobal;
 }
 
@@ -511,7 +489,17 @@ DWindowManagerHelper::MotifDecorations DWindowManagerHelper::getMotifDecorations
 void DWindowManagerHelper::setWmWindowTypes(QWindow *window, WmWindowTypes types)
 {
     const int _types = static_cast<int>(types);
-    QXcbWindowFunctions::setWmWindowType(window, static_cast<QXcbWindowFunctions::WmWindowType>(_types));
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    typedef QNativeInterface::Private::QXcbWindow  QXcbWindow_P;
+    if (auto w = dynamic_cast<QXcbWindow_P *>(window->handle())) {
+        w->setWindowType(static_cast<D_XCB_WINDOW_TYPE>(_types));
+    } else {
+        qWarning() << "cast" << window << "to platform window failed";
+    }
+#else
+    QXcbWindowFunctions::setWmWindowType(window, static_cast<D_XCB_WINDOW_TYPE>(_types));
+#endif
 }
 
 /*!
@@ -524,7 +512,7 @@ void DWindowManagerHelper::setWmWindowTypes(QWindow *window, WmWindowTypes types
 void DWindowManagerHelper::setWmClassName(const QByteArray &name)
 {
     typedef void (*SetWmNameType)(const QByteArray&);
-    return QPlatformHeaderHelper::callPlatformFunction<void, SetWmNameType>(_setWMClassName, name);
+    return callPlatformFunction<void, SetWmNameType>(_setWMClassName, name);
 }
 
 /*!
@@ -536,15 +524,7 @@ void DWindowManagerHelper::setWmClassName(const QByteArray &name)
  */
 void DWindowManagerHelper::popupSystemWindowMenu(const QWindow *window)
 {
-    QFunctionPointer popupSystemWindowMenu = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    popupSystemWindowMenu = qApp->platformFunction(_popupSystemWindowMenu);
-#endif
-
-    if (popupSystemWindowMenu && window->handle()) {
-        reinterpret_cast<void(*)(quint32)>(popupSystemWindowMenu)(window->handle()->winId());
-    }
+    return callPlatformFunction<void, void(*)(quint32)>(_popupSystemWindowMenu, quint32(window->handle()->winId()));
 }
 
 /*!
@@ -553,13 +533,12 @@ void DWindowManagerHelper::popupSystemWindowMenu(const QWindow *window)
  */
 bool DWindowManagerHelper::hasBlurWindow() const
 {
-    QFunctionPointer wmHasBlurWindow = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    wmHasBlurWindow = qApp->platformFunction(_hasBlurWindow);
+#ifndef DTK_DISABLE_TREELAND
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+        return treelandWMHGlobal->hasBlurWindow();
+    }
 #endif
-
-    return wmHasBlurWindow && reinterpret_cast<bool(*)()>(wmHasBlurWindow)();
+    return callPlatformFunction<bool, bool(*)()>(_hasBlurWindow);
 }
 
 /*!
@@ -568,6 +547,12 @@ bool DWindowManagerHelper::hasBlurWindow() const
  */
 bool DWindowManagerHelper::hasComposite() const
 {
+#ifndef DTK_DISABLE_TREELAND
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+        return treelandWMHGlobal->hasComposite();
+    }
+#endif
+
     QFunctionPointer hasComposite = Q_NULLPTR;
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
@@ -588,7 +573,7 @@ bool DWindowManagerHelper::hasComposite() const
         return true;
     }
 
-    return reinterpret_cast<bool(*)()>(hasComposite)();
+    return callPlatformFunction<bool, bool(*)()>(_hasComposite);
 }
 
 /*!
@@ -597,13 +582,12 @@ bool DWindowManagerHelper::hasComposite() const
  */
 bool DWindowManagerHelper::hasNoTitlebar() const
 {
-    QFunctionPointer hasNoTitlebar = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    hasNoTitlebar = qApp->platformFunction(_hasNoTitlebar);
+#ifndef DTK_DISABLE_TREELAND
+    if (DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::IsWaylandPlatform)) {
+        return treelandWMHGlobal->hasNoTitlebar();
+    }
 #endif
-
-    return hasNoTitlebar && reinterpret_cast<bool(*)()>(hasNoTitlebar)();
+    return callPlatformFunction<bool, bool(*)()>(_hasNoTitlebar);
 }
 
 /*!
@@ -612,31 +596,19 @@ bool DWindowManagerHelper::hasNoTitlebar() const
  */
 bool DWindowManagerHelper::hasWallpaperEffect() const
 {
-    QFunctionPointer hasWallpaperEffect = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    hasWallpaperEffect = qApp->platformFunction(_hasWallpaperEffect);
-#endif
-
-    return hasWallpaperEffect && reinterpret_cast<bool(*)()>(hasWallpaperEffect)();
+    return callPlatformFunction<bool, bool(*)()>(_hasWallpaperEffect);
 }
 
 /*!
   \brief DWindowManagerHelper::windowManagerNameString
   \return 返回窗口管理器名称。在X11平台上，此值为窗口管理器对应窗口的 _NET_WM_NAME
   的值
-  \l {https://specifications.freedesktop.org/wm-spec/1.3/ar01s03.html}{_NET_SUPPORTING_WM_CHECK}
-  \l {https://specifications.freedesktop.org/wm-spec/1.3/ar01s05.html}{_NET_WM_NAME}
+  \ref {https://specifications.freedesktop.org/wm-spec/1.3/ar01s03.html}{_NET_SUPPORTING_WM_CHECK}
+  \ref {https://specifications.freedesktop.org/wm-spec/1.3/ar01s05.html}{_NET_WM_NAME}
  */
 QString DWindowManagerHelper::windowManagerNameString() const
 {
-    QFunctionPointer windowManagerName = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    windowManagerName = qApp->platformFunction(_windowManagerName);
-#endif
-
-    return windowManagerName ? reinterpret_cast<QString(*)()>(windowManagerName)() : QString();
+    return callPlatformFunction<QString, QString(*)()>(_windowManagerName);
 }
 
 /*!
@@ -667,15 +639,7 @@ DWindowManagerHelper::WMName DWindowManagerHelper::windowManagerName() const
  */
 QVector<quint32> DWindowManagerHelper::allWindowIdList() const
 {
-    QFunctionPointer wmClientList = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    wmClientList = qApp->platformFunction(_getWindows);
-#endif
-
-    if (!wmClientList) return QVector<quint32>();
-
-    return reinterpret_cast<QVector<quint32>(*)()>(wmClientList)();
+    return callPlatformFunction<QVector<quint32>, QVector<quint32>(*)()>(_getWindows);
 }
 
 /*!
@@ -686,15 +650,7 @@ QVector<quint32> DWindowManagerHelper::allWindowIdList() const
  */
 QVector<quint32> DWindowManagerHelper::currentWorkspaceWindowIdList() const
 {
-    QFunctionPointer wmClientList = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    wmClientList = qApp->platformFunction(_getCurrentWorkspaceWindows);
-#endif
-
-    if (!wmClientList) return QVector<quint32>();
-
-    return reinterpret_cast<QVector<quint32>(*)()>(wmClientList)();
+    return callPlatformFunction<QVector<quint32>, QVector<quint32>(*)()>(_getCurrentWorkspaceWindows);
 }
 
 /*!
@@ -749,16 +705,7 @@ QList<DForeignWindow *> DWindowManagerHelper::currentWorkspaceWindows() const
  */
 quint32 DWindowManagerHelper::windowFromPoint(const QPoint &p)
 {
-    QFunctionPointer winFromPoint = Q_NULLPTR;
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
-    winFromPoint = qApp->platformFunction(_windowFromPoint);
-#endif
-
-    if (!winFromPoint)
-        return 0;
-
-    return reinterpret_cast<quint32 (*)(const QPoint &)>(winFromPoint)(p);
+    return callPlatformFunction<quint32, quint32 (*)(const QPoint &)>(_windowFromPoint, p);
 }
 
 /*!
