@@ -1,29 +1,18 @@
-/*
- * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
- *
- * Author:     zccrs <zccrs@live.com>
- *
- * Maintainer: zccrs <zhangjide@deepin.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
 #ifndef DPLATFORMTHEME_P_H
 #define DPLATFORMTHEME_P_H
 
+#include "private/dplatforminterface_p.h"
 #include "dplatformtheme.h"
 #include "dnativesettings_p.h"
 
+#include <DConfig>
+#include <dguiapplicationhelper.h>
+
+DCORE_USE_NAMESPACE
 DGUI_BEGIN_NAMESPACE
 
 class DPlatformThemePrivate : public DNativeSettingsPrivate
@@ -34,7 +23,6 @@ public:
 
     // 接收parent主题或非调色板DNativeSettings对象（theme对象）的属性变化通知
     // 调色板相关的属性变化与此无关
-    void _q_onThemePropertyChanged(const QByteArray &name, const QVariant &value);
     void onQtColorChanged(QPalette::ColorRole role, const QColor &color);
     void onDtkColorChanged(DPalette::ColorType type, const QColor &color);
     void notifyPaletteChanged();
@@ -50,6 +38,15 @@ public:
     DPalette *palette = nullptr;
     // 减少调色板changed信号的通知频率
     QTimer *notifyPaletteChangeTimer = nullptr;
+    DConfig *dtkPreferenceConfig = nullptr;
+
+    DPlatformInterface *platformInterface = nullptr;
+
+    DGuiApplicationHelper::SizeMode sizeMode = DGuiApplicationHelper::NormalMode;
+    Qt::ScrollBarPolicy scrollBarPolicy = Qt::ScrollBarAsNeeded;
+    
+public slots:
+    void onDtkPreferenceDConfigChanged(const QString &key);
 };
 
 DGUI_END_NAMESPACE
